#include "OgreSceneNodeHierarchyInterface.h"
#include <OgreIteratorWrappers.h>
#include <OgreCamera.h>
#include <OgreSceneNode.h>

namespace Ogre {

//-----------------------------------------------------------------------
SceneNodeHierarchyInterface::SceneNodeHierarchyInterface(SceneManager *sm, RenderSystem *rsys): 
PlatformHierarchyInterface(sm, rsys)
{		
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::TraverseNode(GtpVisibility::HierarchyNode *node)
{
	mNumTraversedNodes ++;
	
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	if (HasGeometry(node))
	{
		RenderNode(node);
	}

	// internal node: add children to priority queue for further processing
	Node::ChildNodeIterator it = sceneNode->getChildIterator();
				
	while (it.hasMoreElements())			
	{ 
		mDistanceQueue->push(it.getNext());
	}
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::RenderNode(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	if (sceneNode->lastRendered() != sceneNode->lastVisited())
	{
		sceneNode->setLastRendered(sceneNode->lastVisited());
		mNumRenderedNodes ++;

		mSceneManager->_renderSceneNode(mCamera, sceneNode, mLeaveTransparentsInQueue);
	}
#endif
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::IsLeaf(GtpVisibility::HierarchyNode *node) const
{
	return (static_cast<SceneNode *>(node)->numChildren() == 0);
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::HasGeometry(GtpVisibility::HierarchyNode *node) const
{
	return static_cast<SceneNode *>(node)->numAttachedObjects() > 0;
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::PullUpVisibility(GtpVisibility::HierarchyNode *node)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);

	while (sceneNode && !sceneNode->isNodeVisible())
	{
		sceneNode->setNodeVisible(true);
		sceneNode = sceneNode->getParentSceneNode();
	}
#endif
}
//-----------------------------------------------------------------------
float SceneNodeHierarchyInterface::GetSquaredDistance(GtpVisibility::HierarchyNode *node) const
{
	return static_cast<SceneNode *>(node)->getSquaredViewDepth(mCamera);
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::SetNodeVisible(GtpVisibility::HierarchyNode *node, 
												 const bool visible)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	sceneNode->setNodeVisible(visible);
#endif
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::SetLastVisited(GtpVisibility::HierarchyNode *node, 
												 const unsigned int frameId)
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	sceneNode->setLastVisited(frameId);
#endif
}
//-----------------------------------------------------------------------
bool SceneNodeHierarchyInterface::IsNodeVisible(GtpVisibility::HierarchyNode *node) const
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	return sceneNode->isNodeVisible();
#else
	return true;
#endif
}
//-----------------------------------------------------------------------
unsigned int SceneNodeHierarchyInterface::LastVisited(GtpVisibility::HierarchyNode *node) const
{
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	SceneNode *sceneNode = static_cast<SceneNode *>(node);
	return sceneNode->lastVisited();
#else
	return 0;
#endif
}
//-----------------------------------------------------------------------
AxisAlignedBox *SceneNodeHierarchyInterface::GetBoundingBox(GtpVisibility::HierarchyNode *node)
{
	// only create renderable bounding box for new node
	if (node != mPreviousNode)
	{
		mPreviousNode = node;
		mBox = static_cast<SceneNode *>(node)->_getWorldAABB();
	}

	return &mBox;
}
//-----------------------------------------------------------------------
void SceneNodeHierarchyInterface::VisualizeCulledNode(GtpVisibility::HierarchyNode *node, 
													  GtpVisibility::CullingType type)
{
	// TODO
}
} // namespace Ogre	
